/***************************************************************************
 * Copyright 2013 Kieker Project (http://kieker-monitoring.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.web.beans.application;

import java.io.Serializable;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.faces.application.FacesMessage.Severity;
import javax.faces.context.FacesContext;

import kieker.common.record.AbstractMonitoringRecord;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

/**
 * This class is a singleton scoped {@code Spring} bean containing properties, constants, methods and even some localized texts which are used at various parts in
 * the application. While some of the properties are loaded directly from the resource bundles, some other are injected via {@code Spring}. This makes sure that most
 * of the constants could be modified without recompiling the whole application.<br>
 * </br>
 * 
 * If you want to change the values, take a look either at the files in the {@code resources} directory or at the {@code Spring} configuration files.<br>
 * </br>
 * 
 * The class implements the {@link Serializable} interface to make sure that some session scoped beans can use this class. It is furthermore singleton scoped as the
 * properties are the same for every user. Therefore only one instance of this class is necessary.<br>
 * </br>
 * 
 * As this bean is {@code Spring} managed, it is <b>not</b> recommended to use the constructor or the setter methods.
 * 
 * @author Nils Christian Ehmke
 */
@Component
@Scope("singleton")
public class GlobalPropertiesBean implements Serializable {

	private static final long serialVersionUID = 4253541090603377504L;

	private static final String RESOURCE_BUNDLE_NAME = "lang.Common";

	private static final String PROPERTY_MSG_PROJECT_LOADING_EXCEPTION = "msgProjectLoadingException";
	private static final String PROPERTY_MSG_PROJECT_SAVED = "msgProjectSaved";
	private static final String PROPERTY_MSG_PLUGIN_CREATION_EXCEPTION = "msgPluginCreationException";
	private static final String PROPERTY_MSG_REPOSITORY_CREATION_EXCEPTION = "msgRepositoryCreationException";
	private static final String PROPERTY_MSG_LIBRARY_UPLOADED = "msgLibraryUploaded";
	private static final String PROPERTY_MSG_PROJECT_CREATED = "msgProjectCreated";
	private static final String PROPERTY_MSG_PROJECT_DELETED = "msgProjectDeleted";
	private static final String PROPERTY_MSG_PROJECT_SAVING_EXCEPTION = "msgProjectSavingException";
	private static final String PROPERTY_MSG_PROJECT_NOT_EXISTING_EXCEPTION = "msgProjectNotExistingException";
	private static final String PROPERTY_MSG_PROJECT_MODIFIED = "msgProjectModified";
	private static final String PROPERTY_MSG_LIBRARY_EXISTING_EXCEPTION = "msgLibraryExistingException";
	private static final String PROPERTY_MSG_LIBRARY_UPLOADING_EXCEPTION = "msgLibraryUploadingException";
	private static final String PROPERTY_NAME_GLOBAL_CONFIGURATION_COMPONENT = "nameGlobalConfigurationComponent";

	private String themeCookieName;
	private String projectOverviewPage;
	private String facesContextThemeKey;
	private String defaultTheme;
	private String analysisEditorGridSizeCookieName;
	private String analysisEditorGridColorCookieName;
	private String analysisEditorDefaultGridSize;
	private String analysisEditorDefaultGridColor;

	/**
	 * Default constructor. <b>Do not use this constructor. This bean is Spring managed.</b>
	 */
	public GlobalPropertiesBean() {
		// No code necessary
	}

	public void setProjectOverviewPage(final String projectOverviewPage) {
		this.projectOverviewPage = projectOverviewPage;
	}

	public String getProjectOverviewPage() {
		return this.projectOverviewPage;
	}

	public void setFacesContextThemeKey(final String facesContextThemeKey) {
		this.facesContextThemeKey = facesContextThemeKey;
	}

	public String getFacesContextThemeKey() {
		return this.facesContextThemeKey;
	}

	public void setDefaultTheme(final String defaultTheme) {
		this.defaultTheme = defaultTheme;
	}

	public String getDefaultTheme() {
		return this.defaultTheme;
	}

	public void setAnalysisEditorGridSizeCookieName(final String analysisEditorGridSizeCookieName) {
		this.analysisEditorGridSizeCookieName = analysisEditorGridSizeCookieName;
	}

	public String getAnalysisEditorGridSizeCookieName() {
		return this.analysisEditorGridSizeCookieName;
	}

	public void setAnalysisEditorGridColorCookieName(final String analysisEditorGridColorCookieName) {
		this.analysisEditorGridColorCookieName = analysisEditorGridColorCookieName;
	}

	public String getAnalysisEditorGridColorCookieName() {
		return this.analysisEditorGridColorCookieName;
	}

	public void setAnalysisEditorDefaultGridSize(final String analysisEditorDefaultGridSize) {
		this.analysisEditorDefaultGridSize = analysisEditorDefaultGridSize;
	}

	public String getAnalysisEditorDefaultGridSize() {
		return this.analysisEditorDefaultGridSize;
	}

	public void setAnalysisEditorDefaultGridColor(final String analysisEditorDefaultGridColor) {
		this.analysisEditorDefaultGridColor = analysisEditorDefaultGridColor;
	}

	public String getAnalysisEditorDefaultGridColor() {
		return this.analysisEditorDefaultGridColor;
	}

	public void setThemeCookieName(final String themeCookieName) {
		this.themeCookieName = themeCookieName;
	}

	public String getThemeCookieName() {
		return this.themeCookieName;
	}

	public String getMsgProjectLoadingException() {
		return GlobalPropertiesBean.getLocalizedString(GlobalPropertiesBean.PROPERTY_MSG_PROJECT_LOADING_EXCEPTION);
	}

	public String getMsgProjectSaved() {
		return GlobalPropertiesBean.getLocalizedString(GlobalPropertiesBean.PROPERTY_MSG_PROJECT_SAVED);
	}

	public String getMsgPluginCreationException() {
		return GlobalPropertiesBean.getLocalizedString(GlobalPropertiesBean.PROPERTY_MSG_PLUGIN_CREATION_EXCEPTION);
	}

	public String getMsgRepositoryCreationException() {
		return GlobalPropertiesBean.getLocalizedString(GlobalPropertiesBean.PROPERTY_MSG_REPOSITORY_CREATION_EXCEPTION);
	}

	public String getMsgLibraryUploaded() {
		return GlobalPropertiesBean.getLocalizedString(GlobalPropertiesBean.PROPERTY_MSG_LIBRARY_UPLOADED);
	}

	public String getMsgProjectCreated() {
		return GlobalPropertiesBean.getLocalizedString(GlobalPropertiesBean.PROPERTY_MSG_PROJECT_CREATED);
	}

	public String getMsgProjectSavingException() {
		return GlobalPropertiesBean.getLocalizedString(GlobalPropertiesBean.PROPERTY_MSG_PROJECT_SAVING_EXCEPTION);
	}

	public String getMsgProjectNotExistingException() {
		return GlobalPropertiesBean.getLocalizedString(GlobalPropertiesBean.PROPERTY_MSG_PROJECT_NOT_EXISTING_EXCEPTION);
	}

	public String getMsgProjectModified() {
		return GlobalPropertiesBean.getLocalizedString(GlobalPropertiesBean.PROPERTY_MSG_PROJECT_MODIFIED);
	}

	public String getMsgLibraryExistingException() {
		return GlobalPropertiesBean.getLocalizedString(GlobalPropertiesBean.PROPERTY_MSG_LIBRARY_EXISTING_EXCEPTION);
	}

	public String getMsgLibraryUploadingException() {
		return GlobalPropertiesBean.getLocalizedString(GlobalPropertiesBean.PROPERTY_MSG_LIBRARY_UPLOADING_EXCEPTION);
	}

	public String getMsgProjectDeleted() {
		return GlobalPropertiesBean.getLocalizedString(GlobalPropertiesBean.PROPERTY_MSG_PROJECT_DELETED);
	}

	public String getGlobalConfigurationComponentName() {
		return GlobalPropertiesBean.getLocalizedString(GlobalPropertiesBean.PROPERTY_NAME_GLOBAL_CONFIGURATION_COMPONENT);
	}

	private static String getLocalizedString(final String key) {
		// Get the correct resource bundle for the current language (taken from the current context) and search it for the given key
		final Locale currLocale = FacesContext.getCurrentInstance().getELContext().getLocale();
		return ResourceBundle.getBundle(GlobalPropertiesBean.RESOURCE_BUNDLE_NAME, currLocale).getString(key);
	}

	/**
	 * This method shows a message by using the growl-component of PrimeFaces (within the current context).
	 * 
	 * @param severity
	 *            The severity of the message.
	 * 
	 * @param msg
	 *            The message to show.
	 */
	public static void showMessage(final Severity severity, final String msg) {
		FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(severity, "", msg));
	}

	/**
	 * This method reads parameters from the request parameter map of the current context. The parameters are converted to actual objects, based on the given types.
	 * It is assumed (but not checked) that both arrays have the same size.
	 * 
	 * @param parameterNames
	 *            The name of the parameters within the request parameter map.
	 * @param types
	 *            The actual types of the parameters. The method will try to convert the strings to those types.
	 * 
	 * @return An array containing the actual parameter objects.
	 */
	public static Object[] convertObjectsFromParameterMap(final String[] parameterNames, final Class<?>[] types) {
		final int numberOfObjects = parameterNames.length;
		final Map<String, String> parameterMap = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap();
		final String[] parameters = new String[numberOfObjects];

		for (int i = 0; i < numberOfObjects; i++) {
			parameters[i] = parameterMap.get(parameterNames[i]);

		}

		// We can borrow a method from the Kieker framework to convert the strings
		return AbstractMonitoringRecord.fromStringArrayToTypedArray(parameters, types);
	}
}
