/***************************************************************************
 * Copyright 2013 Kieker Project (http://kieker-monitoring.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.web.beans.view;

import kieker.analysis.model.analysisMetaModel.MIAnalysisComponent;
import kieker.analysis.model.analysisMetaModel.MIFilter;
import kieker.analysis.model.analysisMetaModel.MIInputPort;
import kieker.analysis.model.analysisMetaModel.MIOutputPort;
import kieker.analysis.model.analysisMetaModel.MIPlugin;
import kieker.analysis.model.analysisMetaModel.MIPort;
import kieker.analysis.model.analysisMetaModel.MIReader;
import kieker.analysis.model.analysisMetaModel.MIRepository;
import kieker.analysis.model.analysisMetaModel.MIRepositoryConnector;
import kieker.common.logging.Log;
import kieker.common.logging.LogFactory;
import kieker.monitoring.core.registry.Registry;
import kieker.webgui.common.exception.GraphLayoutException;
import kieker.webgui.service.IGraphLayoutService;
import kieker.webgui.web.beans.application.GlobalPropertiesBean;
import kieker.webgui.web.beans.session.UserBean;

import org.primefaces.context.RequestContext;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

/**
 * This bean contains the necessary data behind an graph of the analysis editor. It provides various methods to manipulate the current graph.<br>
 * </br>
 * 
 * The class is a Spring managed bean with view scope to make sure that one user (even in one session) can open multiple projects at a time without causing any
 * problems.
 * 
 * @author Nils Christian Ehmke
 */
@Component
@Scope("view")
public class CurrentAnalysisEditorGraphBean {

	private static final Log LOG = LogFactory.getLog(CurrentAnalysisEditorGraphBean.class);

	private static final String[] PARAMETER_NAMES_AUTO_LAYOUT = { "nodes", "edges" };
	private static final Class<?>[] PARAMETER_TYPES_AUTO_LAYOUT = { String.class, String.class };

	private static final String[] PARAMETER_NAMES_ADD_AND_REMOVE_EDGES = { "sourcePortID", "targetPortID" };
	private static final Class<?>[] PARAMETER_TYPES_ADD_AND_REMOVE_EDGES = { String.class, String.class };

	private static final String[] PARAMETER_NAMES_CLICK_AND_REMOVE_NODES = { "ID" };
	private static final Class<?>[] PARAMETER_TYPES_CLICK_AND_REMOVE_NODEST = { String.class };

	/*
	 * The following are a lot of code fragments which are used for the interaction with the flow editor. For details look at the API description of the flow editor
	 * itself.
	 */
	private static final String JS_CMD_CREATE_GRAPH_VARIABLE = "var graph = GraphFlow()";
	private static final String JS_CMD_ADD_EDGE_CONSTRAINTS = "graph.addEdgeConstraints()";

	private static final String JS_CMD_SET_GLOBAL_ICON = "graph.setNodeIcon('Global', '../img/graphIcons/ProjectConfiguration.png', true)";
	private static final String JS_CMD_SET_FILTER_ICON = "graph.setNodeIcon('Filter', '../img/graphIcons/FilterIcon.png', true)";
	private static final String JS_CMD_SET_READER_ICON = "graph.setNodeIcon('Reader', '../img/graphIcons/ReaderIcon.png', true)";
	private static final String JS_CMD_SET_REPOSITORY_ICON = "graph.setNodeIcon('Repository', '../img/graphIcons/RepositoryIcon.png', true)";

	private static final String JS_CMD_ADD_CLICK_NODE_LISTENER = "graph.addListener('onClick', nodeClickListener)";
	private static final String JS_CMD_ADD_ENTER_NODE_LISTENER = "graph.addListener('onMouseEnter', nodeEnterListener)";
	private static final String JS_CMD_ADD_REMOVE_NODE_LISTENER = "graph.addListener('onRemoveNode', nodeRemoveListener)";
	private static final String JS_CMD_ADD_CREATE_EDGE_LISTENER = "graph.addListener('onCreateEdge', edgeCreateListener)";
	private static final String JS_CMD_ADD_REMOVE_EDGE_LISTENER = "graph.addListener('onRemoveEdge', edgeRemoveListener)";
	private static final String JS_CMD_ADD_AUTO_LAYOUT_LISTENER = "graph.addListener('autoLayout', autoLayoutListener)";
	private static final String JS_TEMPLATE_NODE = "{'id':'%s', 'name':'%s', 'nodeClass':'%s', 'tooltip':'%s'}";
	private static final String JS_TEMPLATE_PORT = "{'name':'%s','id':'%s', 'tooltip':'%s'}";

	private static final String JS_CMD_CREATE_GRAPH_CENTER_VARIABLE = "var center = graph.getScreenCenter()";

	private static final Object REPOSITORY_INPUT_PORT = "R";
	private static final String JS_PORT_TYPE_INPUT = "inputPort";

	private static final String JS_CMD_ADD_EDGE = "graph.addEdge('%s', '%s', '%s', false, false)";

	private static final String JS_CMD_ADD_FILTER = JS_CMD_CREATE_GRAPH_CENTER_VARIABLE + "; graph.addNode(center.x,center.y,%s,[%s],[%s],[%s],'Filter',false)";
	private static final String JS_CMD_ADD_READER = JS_CMD_CREATE_GRAPH_CENTER_VARIABLE + "; graph.addNode(center.x,center.y,%s,[%s],null,[%s],'Reader',false)";
	private static final String JS_CMD_ADD_REPOSITORY = JS_CMD_CREATE_GRAPH_CENTER_VARIABLE
			+ "; graph.addNode(center.x,center.y, %s,null, [%s],null,'Repository',false)";
	private static final String JS_CMD_ADD_GLOBAL_COMPONENT = JS_CMD_CREATE_GRAPH_CENTER_VARIABLE
			+ "; graph.addNode(center.x,center.y,{'id':'%s','name':'%s','nodeClass':'','tooltip':'%s'}, null, null, null, 'Global', false)"
			+ "; graph.setNodeData('%s.close', {'$visible' : false});";

	private static final String JS_CMD_ENABLE_GRID = "graph.setGridVisible(true, false)";
	private static final String JS_CMD_DISABLE_GRID = "graph.setGridVisible(false, false)";
	private static final String JS_CMD_ENABLE_SNAP = "graph.setGridSnap(true)";
	private static final String JS_CMD_DISABLE_SNAP = "graph.setGridSnap(false)";
	private static final String JS_CMD_SET_GRID_COLOR = "graph.setGridColor('%s', false)";
	private static final String JS_CMD_SET_GRID_SIZE = "graph.setGridSize(%d, false)";
	private static final String JS_CMD_SET_READ_ONLY = "graph.setReadOnly(false, true)";
	private static final String JS_CMD_SCALE_TO_FIT = "graph.scaleToFit()";

	private static final String JS_CMD_START_AUTO_LAYOUT = "graph.autoLayout()";
	private static final String JS_CMD_LOAD_FROM_LAYOUT = "graph.loadPositions('%s')";
	private static final String JS_CMD_RENAME_NODE = "graph.setNodeData('%s', {'name' : '%s'})";

	/**
	 * We store both instance of MIRepositoryConnector and MIPort in this map. This is necessary as we have to find the correct port when an edge has been created.
	 */
	private final Registry<EObject> portMap = new Registry<EObject>();
	private final Registry<MIAnalysisComponent> componentMap = new Registry<MIAnalysisComponent>();

	private CurrentAnalysisEditorBean currentAnalysisEditorBean;
	private boolean gridEnabled = false;
	private boolean snapEnabled = false;

	@Autowired
	private IGraphLayoutService layouter;
	@Autowired
	private UserBean userBean;
	@Autowired
	private GlobalPropertiesBean globalPropertiesBean;

	/**
	 * Creates a new instance of this class. <b>Do not call this constructor manually. It will only be accessed by Spring.</b>
	 */
	public CurrentAnalysisEditorGraphBean() {
		// No code necessary
	}

	/**
	 * Declares the the graph variable.
	 */
	public void declareGraph() {
		RequestContext.getCurrentInstance().execute(JS_CMD_CREATE_GRAPH_VARIABLE);

		RequestContext.getCurrentInstance().execute(JS_CMD_SET_FILTER_ICON);
		RequestContext.getCurrentInstance().execute(JS_CMD_SET_REPOSITORY_ICON);
		RequestContext.getCurrentInstance().execute(JS_CMD_SET_READER_ICON);
		RequestContext.getCurrentInstance().execute(JS_CMD_SET_GLOBAL_ICON);
	}

	/**
	 * Initializes the listeners for the graph.
	 */
	public void initListeners() {
		RequestContext.getCurrentInstance().execute(JS_CMD_ADD_CLICK_NODE_LISTENER);
		RequestContext.getCurrentInstance().execute(JS_CMD_ADD_ENTER_NODE_LISTENER);

		RequestContext.getCurrentInstance().execute(JS_CMD_ADD_AUTO_LAYOUT_LISTENER);

		RequestContext.getCurrentInstance().execute(JS_CMD_ADD_REMOVE_NODE_LISTENER);
		RequestContext.getCurrentInstance().execute(JS_CMD_ADD_CREATE_EDGE_LISTENER);
		RequestContext.getCurrentInstance().execute(JS_CMD_ADD_REMOVE_EDGE_LISTENER);
	}

	/**
	 * Adds the edge constraints to the graph.
	 */
	public void initEdgeConstraints() {
		RequestContext.getCurrentInstance().execute(CurrentAnalysisEditorGraphBean.JS_CMD_ADD_EDGE_CONSTRAINTS);
	}

	/**
	 * Adds a non removable component to the graph, which can be used to edit project configurations.
	 * 
	 * @param globalConfigurationInstance
	 *            The global configuration instance.
	 */
	public void addGlobalConfigurationInstance(final MIAnalysisComponent globalConfigurationInstance) {
		final String name = this.globalPropertiesBean.getGlobalConfigurationComponentName();
		final int id = this.componentMap.get(globalConfigurationInstance);

		RequestContext.getCurrentInstance().execute(String.format(JS_CMD_ADD_GLOBAL_COMPONENT, "id" + id, name, name, "id" + id));
	}

	/**
	 * This method adds a filter to the current graph.
	 * 
	 * @param filter
	 *            The filter which should be added to the graph.
	 */
	public void addFilter(final MIFilter filter) {
		RequestContext.getCurrentInstance().execute(String.format(JS_CMD_ADD_FILTER, this.assembleGraphString(filter),
				this.assembleGraphRepositoryPortString(filter.getRepositories()), this.assembleGraphInputPortString(filter),
				this.assembleGraphOutputPortString(filter)));
	}

	/**
	 * This method adds a reader to the current graph.
	 * 
	 * @param reader
	 *            The reader which should be added to the graph.
	 */
	public void addReader(final MIReader reader) {
		RequestContext.getCurrentInstance().execute(String.format(JS_CMD_ADD_READER, this.assembleGraphString(reader),
				this.assembleGraphRepositoryPortString(reader.getRepositories()), this.assembleGraphOutputPortString(reader)));
	}

	/**
	 * This method adds a repository to the current graph.
	 * 
	 * @param repository
	 *            The repository which should be added to the graph.
	 */
	public void addRepository(final MIRepository repository) {
		final String repoPort = String.format(JS_TEMPLATE_PORT, JS_PORT_TYPE_INPUT, REPOSITORY_INPUT_PORT, "N/A");
		RequestContext.getCurrentInstance().execute(String.format(JS_CMD_ADD_REPOSITORY, this.assembleGraphString(repository), repoPort));
	}

	/**
	 * This method initializes the auto layout of the graph, resulting in a JavaScript event, delivering the necessary information about the graph.
	 */
	public void startAutoLayout() {
		RequestContext.getCurrentInstance().execute(JS_CMD_START_AUTO_LAYOUT);
	}

	/**
	 * This method uses the given parameters (via request parameter map) to calculate a new layout for the graph and use this layout.
	 */
	public void autoLayout() {
		try {
			// Get the parameters
			final Object[] parameters = GlobalPropertiesBean.convertObjectsFromParameterMap(PARAMETER_NAMES_AUTO_LAYOUT, PARAMETER_TYPES_AUTO_LAYOUT);
			final String nodes = (String) parameters[0];
			final String edges = (String) parameters[1];

			// Calculate the layout and use it
			final String newLayout = this.layouter.layoutGraph(nodes, edges);
			RequestContext.getCurrentInstance().execute(String.format(JS_CMD_LOAD_FROM_LAYOUT, newLayout));
		} catch (final GraphLayoutException ex) {
			CurrentAnalysisEditorGraphBean.LOG.warn("Autolayout failed.", ex);
		}
	}

	/**
	 * This method adds a connection between two filters to the graph.
	 * 
	 * @param source
	 *            The source plugin.
	 * @param destination
	 *            The destination plugin.
	 * @param outputPort
	 *            The output port.
	 * @param inputPort
	 *            The input port.
	 */
	public void addConnection(final MIPlugin source, final MIPlugin destination, final MIOutputPort outputPort, final MIInputPort inputPort) {
		RequestContext.getCurrentInstance().execute(String.format(JS_CMD_ADD_EDGE, this.assembleGraphPortID(source, outputPort),
				this.assembleGraphPortID(destination, inputPort), ""));
	}

	/**
	 * This method adds a connection between a filter and a repository to the graph.
	 * 
	 * @param source
	 *            The source plugin.
	 * @param destination
	 *            The destination repository.
	 * @param port
	 *            The repository port.
	 */
	public void addConnection(final MIPlugin source, final MIRepository destination, final MIRepositoryConnector port) {
		RequestContext.getCurrentInstance().execute(String.format(JS_CMD_ADD_EDGE, this.assembleGraphPortID(source, port),
				this.assembleGraphPortID(destination), ""));
	}

	/**
	 * This is the action which can be called from the javascript code to show that a node has been clicked. It informs the connected
	 * {@link CurrentAnalysisEditorBean} about this.
	 */
	public void nodeClicked() {
		// Get the parameters
		final Object[] parameters = GlobalPropertiesBean.convertObjectsFromParameterMap(PARAMETER_NAMES_CLICK_AND_REMOVE_NODES,
				PARAMETER_TYPES_CLICK_AND_REMOVE_NODEST);
		final String clickedNodeID = (String) parameters[0];

		if (clickedNodeID.isEmpty()) {
			// Deselect
			this.currentAnalysisEditorBean.nodeSelected(null);
		} else {
			// Now search the correct node
			try {
				final MIAnalysisComponent selectedNode = this.componentMap.get(Integer.parseInt(clickedNodeID.substring(2)));
				if ((selectedNode != null) && (this.currentAnalysisEditorBean != null)) {
					this.currentAnalysisEditorBean.nodeSelected(selectedNode);
				}
			} catch (final NumberFormatException ex) {
				// Ignore an invalid ID, but log it.
				CurrentAnalysisEditorGraphBean.LOG.info("Invalid ID", ex);
			}
		}
	}

	/**
	 * This is the action which can be called from the javascript code to show that a node has been removed. It informs the connected
	 * {@link CurrentAnalysisEditorBean} about this.
	 */
	public void nodeRemoved() {
		// Get the parameters

		final Object[] parameters = GlobalPropertiesBean.convertObjectsFromParameterMap(PARAMETER_NAMES_CLICK_AND_REMOVE_NODES,
				PARAMETER_TYPES_CLICK_AND_REMOVE_NODEST);
		final String clickedNodeID = (String) parameters[0];

		// Now search the correct node
		try {
			final MIAnalysisComponent selectedNode = this.componentMap.get(Integer.parseInt(clickedNodeID.substring(2)));
			if ((selectedNode != null) && (this.currentAnalysisEditorBean != null)) {
				this.currentAnalysisEditorBean.nodeRemoved(selectedNode);
			}
		} catch (final NumberFormatException ex) {
			// Ignore an invalid ID, but log it.
			CurrentAnalysisEditorGraphBean.LOG.info("Invalid ID", ex);
		}
	}

	/**
	 * This is the action which can be called from the javascript code to show that an edge has been created. It informs the connected
	 * {@link CurrentAnalysisEditorBean} about this.
	 */
	public void edgeCreated() {
		// Get the parameters
		final Object[] parameters = GlobalPropertiesBean.convertObjectsFromParameterMap(PARAMETER_NAMES_ADD_AND_REMOVE_EDGES, PARAMETER_TYPES_ADD_AND_REMOVE_EDGES);
		final String sourceIDAsStr = ((String) parameters[0]).split("\\.")[0];
		final String sourcePortIDAsStr = ((String) parameters[0]).split("\\.")[1];
		final String targetIDAsStr = ((String) parameters[1]).split("\\.")[0];
		final String targetPortIDAsStr = ((String) parameters[1]).split("\\.")[1];

		try {
			// Convert the IDs
			final int sourceID = Integer.parseInt(sourceIDAsStr.substring(2));
			final int sourcePortID = Integer.parseInt(sourcePortIDAsStr);
			final int targetID = Integer.parseInt(targetIDAsStr.substring(2));

			this.componentMap.get(sourceID);
			final MIAnalysisComponent target = this.componentMap.get(targetID);

			// There are only two allowed cases: A connection between two filters with regular ports and a connection between a filter and a repository using the
			// repository ports.
			if (!REPOSITORY_INPUT_PORT.equals(sourcePortIDAsStr)) {
				// Source is not the special port of a repository
				final EObject sourcePort = this.portMap.get(sourcePortID);
				if (REPOSITORY_INPUT_PORT.equals(targetPortIDAsStr)) {
					// Target is the special port of a repository
					if (sourcePort instanceof MIRepositoryConnector) {
						this.currentAnalysisEditorBean.edgeCreated((MIRepositoryConnector) sourcePort, (MIRepository) target);
					}
				} else {
					// Target is not the special port of a repository
					final int targetPortID = Integer.parseInt(targetPortIDAsStr);
					final EObject targetPort = this.portMap.get(targetPortID);
					if ((sourcePort instanceof MIOutputPort) && (targetPort instanceof MIInputPort)) {
						this.currentAnalysisEditorBean.edgeCreated((MIOutputPort) sourcePort, (MIInputPort) targetPort);
					}
				}
			}
		} catch (final NumberFormatException ex) {
			// Ignore an invalid ID, but log it.
			CurrentAnalysisEditorGraphBean.LOG.info("Invalid ID", ex);
		}
	}

	/**
	 * This is the action which can be called from the javascript code to show that an edge has been removed. It informs the connected
	 * {@link CurrentAnalysisEditorBean} about this.
	 */
	public void edgeRemoved() {
		// Get the parameters
		final Object[] parameters = GlobalPropertiesBean.convertObjectsFromParameterMap(PARAMETER_NAMES_ADD_AND_REMOVE_EDGES, PARAMETER_TYPES_ADD_AND_REMOVE_EDGES);
		final String sourcePortID = ((String) parameters[0]).split("\\.")[1];
		final String targetPortID = ((String) parameters[1]).split("\\.")[1];

		// Now search the correct components
		try {
			if (CurrentAnalysisEditorGraphBean.REPOSITORY_INPUT_PORT.equals(targetPortID)) {
				// This is a special case: An edge between a filter and a repository
				final MIRepositoryConnector sourcePort = (MIRepositoryConnector) this.portMap.get(Integer.parseInt(sourcePortID));
				final String targetID = ((String) parameters[1]).split("\\.")[0];
				final MIAnalysisComponent targetRepo = this.componentMap.get(Integer.parseInt(targetID.substring(2)));
				if ((sourcePort != null) && (targetRepo instanceof MIRepository) && (this.currentAnalysisEditorBean != null)) {
					this.currentAnalysisEditorBean.edgeRemoved(sourcePort, (MIRepository) targetRepo);
				}
			} else {
				// This is the normal case: An edge between two filters
				final MIPort sourcePort = (MIPort) this.portMap.get(Integer.parseInt(sourcePortID));
				final MIPort targetPort = (MIPort) this.portMap.get(Integer.parseInt(targetPortID));

				if ((sourcePort != null) && (targetPort != null) && (this.currentAnalysisEditorBean != null)) {
					this.currentAnalysisEditorBean.edgeRemoved((MIOutputPort) sourcePort, (MIInputPort) targetPort);
				}
			}
		} catch (final NumberFormatException ex) {
			// Ignore an invalid ID, but log it.
			CurrentAnalysisEditorGraphBean.LOG.info("Invalid ID", ex);
		}
	}

	/**
	 * This method sets the color of the grid within the graph.
	 * 
	 * @param color
	 *            The new color as a HTML color (FF0080 e.g.).
	 */
	public void setGridColor(final String color) {
		final String cmd = String.format(CurrentAnalysisEditorGraphBean.JS_CMD_SET_GRID_COLOR, "#" + color);
		RequestContext.getCurrentInstance().execute(cmd);
	}

	/**
	 * This method sets the size of the grid within the graph.
	 * 
	 * @param size
	 *            The new grid size.
	 */
	public void setGridSize(final int size) {
		final String cmd = String.format(CurrentAnalysisEditorGraphBean.JS_CMD_SET_GRID_SIZE, size);
		RequestContext.getCurrentInstance().execute(cmd);
	}

	/**
	 * This method scales the graph to fit the current view. After this method-call, all components of the graph should be visible.
	 */
	public void scaleToFit() {
		RequestContext.getCurrentInstance().execute(CurrentAnalysisEditorGraphBean.JS_CMD_SCALE_TO_FIT);
	}

	/**
	 * This method switches the visibility of the grid within the graph.
	 */
	public void switchGrid() {
		if (this.gridEnabled) {
			RequestContext.getCurrentInstance().execute(CurrentAnalysisEditorGraphBean.JS_CMD_DISABLE_GRID);
		} else {
			RequestContext.getCurrentInstance().execute(CurrentAnalysisEditorGraphBean.JS_CMD_ENABLE_GRID);
		}

		this.gridEnabled = this.gridEnabled ^ true; // this.gridEnabled = !this.gridEnabled;
	}

	/**
	 * This method switches the grid snap within the graph.
	 */
	public void switchSnap() {
		if (this.snapEnabled) {
			RequestContext.getCurrentInstance().execute(CurrentAnalysisEditorGraphBean.JS_CMD_DISABLE_SNAP);
		} else {
			RequestContext.getCurrentInstance().execute(CurrentAnalysisEditorGraphBean.JS_CMD_ENABLE_SNAP);
		}

		this.snapEnabled = this.snapEnabled ^ true; // this.snapEnabled = !this.snapEnabled;
	}

	/**
	 * Renames a given node and repaints the graph.
	 * 
	 * @param node
	 *            The node to rename.
	 * @param newName
	 *            The new name of the node.
	 */
	public void renameNode(final MIAnalysisComponent node, final String newName) {
		final String cmd = String.format(CurrentAnalysisEditorGraphBean.JS_CMD_RENAME_NODE, "id" + this.componentMap.get(node),
				CurrentAnalysisEditorGraphBean.simpleEscape(newName));
		RequestContext.getCurrentInstance().execute(cmd);
	}

	/**
	 * This method checks whether the current user is a guest and activates the read only mode if necessary.
	 */
	public void checkReadOnlyForGuests() {
		if ("Guest".equals(this.userBean.getUserrole())) {
			RequestContext.getCurrentInstance().execute(JS_CMD_SET_READ_ONLY);
		}
	}

	public boolean isGridEnabled() {
		return this.gridEnabled;
	}

	public boolean isSnapEnabled() {
		return this.snapEnabled;
	}

	public void setCurrentAnalysisEditorBean(final CurrentAnalysisEditorBean currentAnalysisEditorBean) {
		this.currentAnalysisEditorBean = currentAnalysisEditorBean;
	}

	/**
	 * This method makes sure that the given layout will be loaded by the current graph.
	 * 
	 * @param layout
	 *            The new layout for the graph.
	 */
	public void loadLayout(final String layout) {
		RequestContext.getCurrentInstance().execute(String.format(JS_CMD_LOAD_FROM_LAYOUT, layout));
	}

	/**
	 * This method assembles the string containing the given ports.
	 * 
	 * @param ports
	 *            The ports to be used within the string command.
	 * @return A string containing the JS commands to create the ports.
	 */
	private String assembleGraphPortString(final EList<? extends MIPort> ports) {
		final StringBuilder builder = new StringBuilder();
		final int len = ports.size();

		for (int i = 0; i < len; i++) {
			final MIPort port = ports.get(i);

			if (i != 0) {
				builder.append(',');
			}

			builder.append(String.format(JS_TEMPLATE_PORT, CurrentAnalysisEditorGraphBean.simpleEscape(port.getName()), this.portMap.get(port),
					CurrentAnalysisEditorGraphBean.simpleEscape(port.getName())));
		}

		return builder.toString();

	}

	/**
	 * This method assembles the string containing the available repository ports of the given plugin.
	 * 
	 * @param ports
	 *            The ports which will be used.
	 * @return A string containing the JS commands to create the repository ports.
	 */
	private String assembleGraphRepositoryPortString(final EList<MIRepositoryConnector> ports) {
		final StringBuilder builder = new StringBuilder();
		final int len = ports.size();

		for (int i = 0; i < len; i++) {
			final MIRepositoryConnector port = ports.get(i);

			if (i != 0) {
				builder.append(',');
			}

			builder.append(String.format(JS_TEMPLATE_PORT, CurrentAnalysisEditorGraphBean.simpleEscape(port.getName()), this.portMap.get(port),
					CurrentAnalysisEditorGraphBean.simpleEscape(port.getName())));
		}

		return builder.toString();
	}

	/**
	 * This method assembles the string containing the available input ports of the given filter.
	 * 
	 * @param filter
	 *            The filter whose input ports will be used.
	 * @return A string containing the JS commands to create the input ports.
	 */
	private String assembleGraphInputPortString(final MIFilter filter) {
		return this.assembleGraphPortString(filter.getInputPorts());
	}

	/**
	 * This method assembles the string containing the available output ports of the given plugin.
	 * 
	 * @param plugin
	 *            The plugin whose output ports will be used.
	 * @return A string containing the JS commands to create the output ports.
	 */
	private String assembleGraphOutputPortString(final MIPlugin plugin) {
		return this.assembleGraphPortString(plugin.getOutputPorts());
	}

	/**
	 * This method assembles the ID of the port for the graph based on the given parameters.
	 * 
	 * @param plugin
	 *            The parent plugin of the port.
	 * @param port
	 *            The port itself.
	 * @return The ID for the port within the graph
	 */
	private String assembleGraphPortID(final MIPlugin plugin, final MIPort port) {
		return "id" + this.componentMap.get(plugin) + "." + this.portMap.get(port);
	}

	/**
	 * This method assembles the ID of the port for the graph based on the given parameters.
	 * 
	 * @param plugin
	 *            The parent plugin of the port.
	 * @param port
	 *            The port itself.
	 * @return The ID for the port within the graph
	 */
	private Object assembleGraphPortID(final MIPlugin plugin, final MIRepositoryConnector port) {
		return "id" + this.componentMap.get(plugin) + "." + this.portMap.get(port);
	}

	/**
	 * This method assembles the ID of the port for the graph based on the given parameters.
	 * 
	 * @param repository
	 *            The parent repository of the port.
	 * @return The ID for the port within the graph
	 */
	private Object assembleGraphPortID(final MIRepository repository) {
		return "id" + this.componentMap.get(repository) + "." + REPOSITORY_INPUT_PORT;
	}

	/**
	 * Assembles a human-readable string of the given repository or plugin, which can also be used as a graph ID.
	 * 
	 * @param component
	 *            The component whose ID should be delivered.
	 * @return A human readable ID.
	 */
	private String assembleGraphString(final MIAnalysisComponent component) {
		final String name = component.getName();
		final String className = component.getClassname();
		final String shortName = className.substring(className.lastIndexOf('.') + 1);

		return String.format(JS_TEMPLATE_NODE, "id" + this.componentMap.get(component),
				CurrentAnalysisEditorGraphBean.simpleEscape(name), CurrentAnalysisEditorGraphBean.simpleEscape(shortName),
				CurrentAnalysisEditorGraphBean.simpleEscape(className));
	}

	/**
	 * This method escapes the given string by removing the character ' from the string. This is necessary as the character could destroy the flow editor.
	 * 
	 * @param str
	 *            The string to be escaped.
	 * @return The escaped string.
	 */
	private static final String simpleEscape(final String str) {
		if (str != null) {
			return str.replaceAll("\'", "");
		} else {
			return str;
		}
	}

}
